import React, { useState, useEffect, useRef } from 'react';
// import axios from 'axios';
import { Link, useNavigate } from 'react-router-dom';
import { PRIMARY_COLOR, SECONDARY_COLOR } from '../App';
import './Admin.css';
import { useApiCall } from './hooks/useApiCall';
import { api } from '../services/api';
import { 
  Typography, 
  Table, 
  TableBody, 
  TableCell, 
  TableContainer, 
  TableHead, 
  TableRow, 
  Paper, 
  Button, 
  Alert,
  Container,
  Box,
  TextField,
  CircularProgress,
  Tab,
  Tabs
} from '@mui/material';

export default function Admin() {
  const [users, setUsers] = useState([]);
  const [tags, setTags] = useState([]);
  const [username, setUsername] = useState('');
  const [email, setEmail] = useState('');
  const [password, setPassword] = useState('');
  const [activeTab, setActiveTab] = useState(0);
  const navigate = useNavigate();
  const fileInputRef = useRef(null);

  const { execute: fetchUsers, loading: loadingUsers, error: usersError } = useApiCall();
  const { execute: fetchTags, loading: loadingTags, error: tagsError } = useApiCall();
  const { execute: createUser, loading: creatingUser, error: createError } = useApiCall();
  const { execute: deleteUser, loading: deletingUser, error: deleteError } = useApiCall();
  const { execute: deleteTag, loading: deletingTag, error: deleteTagError } = useApiCall();
  const { execute: backupDB, loading: backingUp, error: backupError } = useApiCall();
  const { execute: restoreDB, loading: restoring, error: restoreError } = useApiCall();

  useEffect(() => {
    const getInitialData = async () => {
      try {
        const [usersResponse, tagsResponse] = await Promise.all([
          fetchUsers(() => api.admin.getUsers(localStorage.getItem('token'))),
          fetchTags(() => api.tags.getAll(localStorage.getItem('token')))
        ]);
        setUsers(usersResponse.data);
        setTags(tagsResponse.data);
      } catch (err) {}
    };
    getInitialData();
  }, []);

  const handleCreateUser = async (e) => {
    e.preventDefault();
    try {
      const response = await createUser(() => 
        api.admin.createUser(
          localStorage.getItem('token'),
          { username, password, email }
        )
      );
      setUsers([...users, response.data]);
      setUsername('');
      setPassword('');
      setEmail('');
    } catch (err) {}
  };

  const handleDeleteUser = async (id) => {
    try {
      await deleteUser(() => 
        api.admin.deleteUser(localStorage.getItem('token'), id)
      );
      setUsers(users.filter(user => user.id !== id));
    } catch (err) {}
  };

  const handleDeleteTag = async (tagId) => {
    if (!window.confirm('Are you sure you want to delete this tag? This will remove it from all items.')) {
      return;
    }
    
    try {
      await deleteTag(() => 
        api.tags.delete(localStorage.getItem('token'), tagId)
      );
      setTags(tags.filter(tag => tag.ID !== tagId));
    } catch (err) {}
  };

  const handleBackupDatabase = async () => {
    try {
      const response = await backupDB(() => 
        api.admin.backupDB(localStorage.getItem('token'))
      );
      const blob = new Blob([response.data], { type: 'application/x-sqlite3' });
      const url = URL.createObjectURL(blob);
      const a = document.createElement('a');
      a.href = url;
      a.download = 'database.db';
      a.click();
    } catch (err) {}
  };

  const handleRestoreDatabase = async (e) => {
    e.preventDefault();
    if (!fileInputRef.current?.files?.[0]) return;

    try {
      const formData = new FormData();
      formData.append('database', fileInputRef.current.files[0]);

      await restoreDB(() => 
        api.admin.restoreDB(localStorage.getItem('token'), formData)
      );
      alert('Database restored successfully');
      navigate('/admin');
    } catch (err) {}
  };

  return (
    <Container maxWidth="md">
      <Typography variant="h4" gutterBottom>
        Admin Dashboard
      </Typography>

      {(usersError || createError || deleteError || backupError || restoreError || tagsError || deleteTagError) && (
        <Alert severity="error" sx={{ mb: 2 }}>
          {usersError?.message || createError?.message || deleteError?.message || 
           backupError?.message || restoreError?.message || tagsError?.message || deleteTagError?.message}
        </Alert>
      )}

      <Tabs value={activeTab} onChange={(e, newValue) => setActiveTab(newValue)} sx={{ mb: 3 }}>
        <Tab label="Users" />
        <Tab label="Tags" />
        <Tab label="Database" />
      </Tabs>

      {activeTab === 0 && (
        <>
          <Box component="form" onSubmit={handleCreateUser} sx={{ mb: 4 }}>
            <Typography variant="h6" gutterBottom>
              Add New User
            </Typography>
            <TextField
              label="Username"
              variant="outlined"
              value={username}
              onChange={(e) => setUsername(e.target.value)}
              sx={{ mr: 2 }}
              disabled={creatingUser}
            />
            <TextField
              label="Password"
              type="password"
              variant="outlined"
              value={password}
              onChange={(e) => setPassword(e.target.value)}
              sx={{ mr: 2 }}
              disabled={creatingUser}
            />
            <TextField
              label="Email"
              variant="outlined"
              value={email}
              onChange={(e) => setEmail(e.target.value)}
              sx={{ mr: 2 }}
              disabled={creatingUser}
            />
            <Button 
              type="submit" 
              sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} 
              variant="contained"
              disabled={creatingUser}
            >
              {creatingUser ? <CircularProgress size={24} /> : 'Add User'}
            </Button>
          </Box>

          {loadingUsers ? (
            <CircularProgress />
          ) : (
            <TableContainer component={Paper}>
              <Table>
                <TableHead>
                  <TableRow>
                    <TableCell>ID</TableCell>
                    <TableCell>Username</TableCell>
                    <TableCell>Email</TableCell>
                    <TableCell>Actions</TableCell>
                  </TableRow>
                </TableHead>
                <TableBody>
                  {users.map(user => (
                    <TableRow key={user.ID}>
                      <TableCell>{user.ID}</TableCell>
                      <TableCell>{user.username}</TableCell>
                      <TableCell>{user.email}</TableCell>
                      <TableCell>
                        <Button 
                          variant="contained" 
                          color="error" 
                          onClick={() => handleDeleteUser(user.ID)}
                          disabled={deletingUser}
                        >
                          {deletingUser ? <CircularProgress size={24} /> : 'Delete User'}
                        </Button>
                      </TableCell>
                    </TableRow>
                  ))}
                </TableBody>
              </Table>
            </TableContainer>
          )}
        </>
      )}

      {activeTab === 1 && (
        <>
          <Typography variant="h6" gutterBottom>
            Manage Tags
          </Typography>
          {loadingTags ? (
            <CircularProgress />
          ) : (
            <TableContainer component={Paper}>
              <Table>
                <TableHead>
                  <TableRow>
                    <TableCell>Color</TableCell>
                    <TableCell>Name</TableCell>
                    <TableCell>Description</TableCell>
                    <TableCell>Actions</TableCell>
                  </TableRow>
                </TableHead>
                <TableBody>
                  {tags.map(tag => (
                    <TableRow key={tag.ID}>
                      <TableCell>
                        <Box
                          sx={{
                            width: 24,
                            height: 24,
                            backgroundColor: tag.color,
                            borderRadius: '50%'
                          }}
                        />
                      </TableCell>
                      <TableCell>{tag.name}</TableCell>
                      <TableCell>{tag.description}</TableCell>
                      <TableCell>
                        <Button 
                          variant="contained" 
                          color="error" 
                          onClick={() => handleDeleteTag(tag.ID)}
                          disabled={deletingTag}
                        >
                          {deletingTag ? <CircularProgress size={24} /> : 'Delete Tag'}
                        </Button>
                      </TableCell>
                    </TableRow>
                  ))}
                </TableBody>
              </Table>
            </TableContainer>
          )}
        </>
      )}

      {activeTab === 2 && (
        <Box sx={{ mt: 4 }}>
          <Typography variant="h6" gutterBottom>
            Database Management
          </Typography>
          <Button 
            variant="contained" 
            onClick={handleBackupDatabase}
            sx={{ mr: 2, backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
            disabled={backingUp}
          >
            {backingUp ? <CircularProgress size={24} /> : 'Backup Database'}
          </Button>

          <Button 
            variant="contained" 
            component="label"
            sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
            disabled={restoring}
          >
            {restoring ? <CircularProgress size={24} /> : 'Restore Database'}
            <input
              type="file"
              hidden
              ref={fileInputRef}
              onChange={handleRestoreDatabase}
            />
          </Button>
        </Box>
      )}
    </Container>
  );
}