import React, { useEffect, useState, useRef, useCallback, useMemo } from 'react';
import { 
  Container, 
  Button,
  IconButton,
  TextField,
  TableContainer,
  Table,
  TableHead,
  TableRow,
  TableCell,
  TableBody,
  Box,
  Tooltip,
  Avatar,
  Dialog,
  DialogTitle,
  DialogContent,
  DialogActions,
  Alert,
  CircularProgress,
  Chip,
  Typography,
  FormGroup
} from '@mui/material';
import { Delete as DeleteIcon, Edit as EditIcon } from '@mui/icons-material';
import { useParams, useLocation } from 'react-router-dom';
import { PRIMARY_COLOR, SECONDARY_COLOR } from '../App';
import { useApiCall } from './hooks/useApiCall';
import { api } from '../services/api';
import ItemDetails from './ItemDetails';
import TagManager from './tagManager';

const Item = React.memo(({ item, onDelete, onEdit, itemImages }) => (
  <TableRow>
    <TableCell>
      <Avatar src={itemImages[item.ID] || '/images/default.jpg'} />
    </TableCell>
    <TableCell>{item.name}</TableCell>
    <TableCell>{item.description}</TableCell>
    <TableCell>
      {item.tags?.map(tag => (
        <Chip
          key={tag.ID}
          label={tag.name}
          size="small"
          style={{ 
            backgroundColor: tag.color,
            color: '#fff',
            margin: '0 2px'
          }}
        />
      ))}
    </TableCell>
    <TableCell>
      <Box display="flex" justifyContent="space-between" width="100%">
        <Tooltip title="Edit Item">
          <IconButton onClick={() => onEdit(item)} size="large" sx={{ mr: 1 }}>
            <EditIcon />
          </IconButton>
        </Tooltip>
        <Tooltip title="Delete Item">
          <IconButton onClick={() => onDelete(item.ID)} size="large" color="error">
            <DeleteIcon />
          </IconButton>
        </Tooltip>
      </Box>
    </TableCell>
  </TableRow>
));

export default function Items({ token }) {
  const { id: boxId } = useParams();
  const [items, setItems] = useState([]);
  const [newItemName, setNewItemName] = useState('');
  const [newItemDescription, setNewItemDescription] = useState('');
  const [editingItem, setEditingItem] = useState(null);
  const location = useLocation();
  const boxName = location.state?.boxName || 'All Boxes';
  const [itemImages, setItemImages] = useState({});
  const fileInputRef = useRef(null);
  const [openAddItemDialog, setOpenAddItemDialog] = useState(false);
  const [searchQuery, setSearchQuery] = useState('');
  const [availableTags, setAvailableTags] = useState([]);
  const [selectedTags, setSelectedTags] = useState([]);
  
  const { execute: fetchItems, loading: loadingItems, error: itemsError } = useApiCall();
  const { execute: fetchTags, loading: loadingTags, error: tagsError } = useApiCall();
  const { execute: createItem, loading: creatingItem, error: createError } = useApiCall();
  const { execute: deleteItem, loading: deletingItem, error: deleteError } = useApiCall();
  const { execute: uploadImage, loading: uploadingImage, error: uploadError } = useApiCall();

  // Fetch items and tags on component mount
  useEffect(() => {
    const loadData = async () => {
      try {
        // Fetch items and tags in parallel
        const [itemsResponse, tagsResponse] = await Promise.all([
          fetchItems(() => boxId ? api.items.getByBox(token, boxId) : api.items.getAll(token)),
          fetchTags(() => api.tags.getAll(token))
        ]);

        setItems(itemsResponse.data);
        setAvailableTags(tagsResponse.data);

        // Fetch images for each item
        itemsResponse.data.forEach(item => {
          api.items.getImage(token, item.ID)
            .then(response => {
              const reader = new FileReader();
              reader.onload = () => {
                setItemImages(prev => ({
                  ...prev,
                  [item.ID]: reader.result
                }));
              };
              reader.readAsDataURL(response.data);
            })
            .catch(() => {
              setItemImages(prev => ({
                ...prev,
                [item.ID]: '/default.jpg'
              }));
            });
        });
      } catch (err) {}
    };
    loadData();
  }, [token, boxId]);

  // Filter items based on search query and selected tags
  const filteredItems = useMemo(() => {
    return items.filter(item => {
      // Text search match
      const textMatch = 
        item.name.toLowerCase().includes(searchQuery.toLowerCase()) ||
        item.description.toLowerCase().includes(searchQuery.toLowerCase());

      // Tag match - if no tags selected, show all items
      const tagMatch = selectedTags.length === 0 || 
        selectedTags.every(tagId => 
          item.tags?.some(itemTag => itemTag.ID === tagId)
        );

      return textMatch && tagMatch;
    });
  }, [items, searchQuery, selectedTags]);

  // Handle tag selection
  const handleTagSelect = (tagId) => {
    setSelectedTags(prev => 
      prev.includes(tagId)
        ? prev.filter(id => id !== tagId)
        : [...prev, tagId]
    );
  };

  // Rest of the component remains the same...
  // (keeping all existing functions like handleAddItem, handleDeleteItem, etc.)
  const handleAddItem = useCallback(() => {
    setOpenAddItemDialog(true);
  }, []);

  const handleCloseAddItemDialog = useCallback(() => {
    setOpenAddItemDialog(false);
    setNewItemName('');
    setNewItemDescription('');
    if (fileInputRef.current) {
      fileInputRef.current.value = '';
    }
  }, []);

  const generateUniqueImageName = useCallback((imageName) => {
    if (imageName.toLowerCase() === 'image.jpg') {
      const randomString = Math.random().toString(36).substr(2, 9);
      return `image_${randomString}.jpg`;
    }
    return imageName;
  }, []);
  const handleDeleteItem = useCallback(async (itemId) => {
    try {
      await deleteItem(() => api.items.delete(token, itemId));
      setItems(prev => prev.filter(item => item.ID !== itemId));
    } catch (err) {}
  }, [token, deleteItem]);

  const handleEditItem = useCallback((item) => {
    setEditingItem(item);
  }, []);

  const handleSaveEdit = useCallback(async () => {
    setEditingItem(null);
    // Refresh the items list after editing
    try {
      const response = await fetchItems(() => 
        boxId ? api.items.getByBox(token, boxId) : api.items.getAll(token)
      );
      setItems(response.data);
    } catch (err) {}
  }, [token, boxId, fetchItems]);
  
  const handleSaveNewItem = useCallback(async () => {
    try {
      const newItemResponse = await createItem(() => 
        api.items.create(token, {
          name: newItemName,
          description: newItemDescription,
          box_id: parseInt(boxId, 10)
        })
      );
  
      if (newItemResponse.status === 200 && fileInputRef.current?.files?.[0]) {
        const imageFile = fileInputRef.current.files[0];
        const newImageName = generateUniqueImageName(imageFile.name);
        const formData = new FormData();
        formData.append('image', new File([imageFile], newImageName, {
          type: imageFile.type,
        }));
  
        await uploadImage(() => 
          api.items.uploadImage(token, newItemResponse.data.id, formData)
        );
        if (newItemResponse.data.id) {
          try {
            const imageResponse = await api.items.getImage(token, newItemResponse.data.id);
            const reader = new FileReader();
            reader.onload = () => {
              setItemImages(prev => ({
                ...prev,
                [newItemResponse.data.id]: reader.result
              }));
            };
            reader.readAsDataURL(imageResponse.data);
          } catch (err) {
            setItemImages(prev => ({
              ...prev,
              [newItemResponse.data.id]: '/default.jpg'
            }));
          }
        }
      }
  
      handleCloseAddItemDialog();
      
      const response = await fetchItems(() => 
        boxId ? api.items.getByBox(token, boxId) : api.items.getAll(token)
      );
      setItems(response.data);
    } catch (err) {}
  }, [token, boxId, newItemName, newItemDescription, createItem, uploadImage, fetchItems, generateUniqueImageName]);

  return (
    <Container>
      {(itemsError || tagsError || createError || deleteError || uploadError) && (
        <Alert severity="error" sx={{ mb: 2 }}>
          {itemsError?.message || tagsError?.message || createError?.message || 
           deleteError?.message || uploadError?.message}
        </Alert>
      )}
  
      <Box sx={{ mb: 3 }}>
        <TextField
          label="Search items..."
          variant="outlined"
          fullWidth
          margin="normal"
          value={searchQuery}
          onChange={(e) => setSearchQuery(e.target.value)}
        />
  
        <Box sx={{ mt: 2, mb: 2 }}>
          <Typography variant="subtitle1" gutterBottom>Filter by tags:</Typography>
          <Box display="flex" flexWrap="wrap" gap={1}>
            {availableTags.map(tag => (
              <Chip
                key={tag.ID}
                label={tag.name}
                style={{
                  backgroundColor: selectedTags.includes(tag.ID) ? tag.color : 'transparent',
                  color: selectedTags.includes(tag.ID) ? '#fff' : 'inherit',
                  border: `1px solid ${tag.color}`
                }}
                onClick={() => handleTagSelect(tag.ID)}
                clickable
              />
            ))}
          </Box>
        </Box>
      </Box>
  
      <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center', mb: 2 }}>
        <Typography variant="h6">
          {boxName === "All Boxes" ? "All Items" : `Items in ${boxName}`} 
          ({filteredItems.length} items)
        </Typography>
  
        <Button 
          sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }}
          variant="contained"
          onClick={handleAddItem}
          disabled={creatingItem}
        >
          {creatingItem ? <CircularProgress size={24} /> : 'Add Item'}
        </Button>
      </Box>
  
      <Dialog open={openAddItemDialog} onClose={handleCloseAddItemDialog}>
        <DialogTitle>Add New Item</DialogTitle>
        <DialogContent>
          <TextField
            label="Item Name"
            variant="outlined"
            fullWidth
            margin="normal"
            value={newItemName}
            onChange={(e) => setNewItemName(e.target.value)}
          />
          <TextField
            label="Item Description"
            variant="outlined"
            fullWidth
            margin="normal"
            value={newItemDescription}
            onChange={(e) => setNewItemDescription(e.target.value)}
          />
          <input
            type="file"
            accept="image/*"
            ref={fileInputRef}
            style={{ display: 'block', margin: '10px 0' }}
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleCloseAddItemDialog}>Cancel</Button>
          <Button 
            onClick={handleSaveNewItem} 
            color="primary"
            disabled={creatingItem || !newItemName.trim()}
          >
            {creatingItem ? <CircularProgress size={24} /> : 'Save'}
          </Button>
        </DialogActions>
      </Dialog>
  
      {editingItem && (
        <ItemDetails 
          item={editingItem} 
          token={token} 
          onSave={handleSaveEdit} 
          onClose={() => setEditingItem(null)} 
          boxId={boxId}
        />
      )}
  
      {loadingItems ? (
        <CircularProgress />
      ) : (
        <TableContainer>
          <Table>
            <TableHead>
              <TableRow>
                <TableCell style={{ width: '40px' }}>Image</TableCell>
                <TableCell style={{ width: '100px' }}>Name</TableCell>
                <TableCell>Description</TableCell>
                <TableCell>Tags</TableCell>
                <TableCell>Actions</TableCell>
              </TableRow>
            </TableHead>
            <TableBody>
              {filteredItems.map((item) => (
                <Item 
                  key={item.ID}
                  item={item}
                  itemImages={itemImages}
                  onDelete={handleDeleteItem}
                  onEdit={handleEditItem}
                />
              ))}
            </TableBody>
          </Table>
        </TableContainer>
      )}
    </Container>
  )};